/*
 *  CardOSError.cpp
 *
 *  Copyright (c) 2011 Atos IT Solutions and Services GmbH
 *
 *  CardOS TokenD implementation based on XYZError.cpp provided 
 *  as part of the Apple TokenD developer information available from 
 *  dts@apple.com .
 *
 *  Use diff -c XYZError.cpp CardOSError.cpp to generate a patch.
 *
 *  See below for the copyright notice of the original file
 *  and the license applying to both the original and modified code.
 *
 *  Copyright (c) 2004 Apple Computer, Inc. All Rights Reserved.
 * 
 *  @APPLE_LICENSE_HEADER_START@
 *  
 *  This file contains Original Code and/or Modifications of Original Code
 *  as defined in and that are subject to the Apple Public Source License
 *  Version 2.0 (the 'License'). You may not use this file except in
 *  compliance with the License. Please obtain a copy of the License at
 *  http://www.opensource.apple.com/apsl/ and read it before using this
 *  file.
 *  
 *  The Original Code and all software distributed under the License are
 *  distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
 *  EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
 *  INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
 *  Please see the License for the specific language governing rights and
 *  limitations under the License.
 *  
 *  @APPLE_LICENSE_HEADER_END@
 */



#include "CardOSError.h"

#include <Security/cssmerr.h>

//
// CardOSError exceptions
//
CardOSError::CardOSError(CSSM_RETURN err) : CssmError(err)
{
  IFDEBUG(debugDiagnose(this));
}

const char *CardOSError::what() const throw ()
{
  return "CardOSError";
}


void CardOSError::check(const net_atos_cardos_api::CardOSAPI_Exception & cardosEx)
{ 
  if (cardosEx.getErrorCode() != CARDOS_API_OK) throwMe(cardosEx); 
}


void CardOSError::throwMe(const net_atos_cardos_api::CardOSAPI_Exception & cardosEx)
{
  CSSM_RETURN err;
  
  switch (cardosEx.getErrorCode()) {
	case CARDOS_API_E_MEMORY_ERROR:
	  err = CSSM_ERRCODE_MEMORY_ERROR;
	  break;
	case CARDOS_API_E_INVALID_INPUT_POINTER: 
	  err = CSSM_ERRCODE_INVALID_INPUT_POINTER;
	  break;
	case CARDOS_API_E_FUNCTION_FAILED:
	  err = CSSM_ERRCODE_FUNCTION_FAILED;
	  break;
	case CARDOS_API_E_OPERATION_AUTH_DENIED:
	  err = CSSM_ERRCODE_OPERATION_AUTH_DENIED;
	break;
    case CARDOS_API_E_INVALID_SAMPLE_VALUE:
	  err = CSSM_ERRCODE_INVALID_SAMPLE_VALUE;
      break;
	case CARDOS_API_E_INVALID_CONTEXT_HANDLE:
	  err = CSSM_ERRCODE_INVALID_CONTEXT_HANDLE;
	  break;
	case CARDOS_API_E_INVALID_KEY:
	  err = CSSMERR_CSP_INVALID_KEY;
	  break;
	case CARDOS_API_E_INVALID_KEY_REFERENCE:
	  err = CSSMERR_CSP_INVALID_KEY_REFERENCE;
	  break;
	case CARDOS_API_E_INVALID_KEY_LABEL:
	  err = CSSMERR_CSP_INVALID_KEY_LABEL;
      break;
	case CARDOS_API_E_NOT_LOGGED_IN:
	  err = CSSMERR_CSP_NOT_LOGGED_IN;
	  break;
	case CARDOS_API_E_INTERNAL_ERROR:
	default:
      err = CSSM_ERRCODE_INTERNAL_ERROR;
      break;
  }
  
  throw CardOSError(err);
}


#if !defined(NDEBUG)

void CardOSError::debugDiagnose(const void *id) const
{
  secdebug("exception", "CardOSToken::%p CardOS Error %s (%04hX)",
           id, errorstr(statusWord), statusWord);
}

#endif //NDEBUG
