/*
 *  CardOSRecord.cpp
 *
 *  Copyright (c) 2011 Atos IT Solutions and Services GmbH
 *
 *  CardOS TokenD implementation based on XYZRecord.cpp provided 
 *  as part of the Apple TokenD developer information available from 
 *  dts@apple.com .
 *
 *  Use diff -c XYZRecord.cpp CardOSRecord.cpp to generate a patch.
 *
 *  See below for the copyright notice of the original file
 *  and the license applying to both the original and modified code.
 *
 *  Copyright (c) 2004 Apple Computer, Inc. All Rights Reserved.
 * 
 *  @APPLE_LICENSE_HEADER_START@
 *  
 *  This file contains Original Code and/or Modifications of Original Code
 *  as defined in and that are subject to the Apple Public Source License
 *  Version 2.0 (the 'License'). You may not use this file except in
 *  compliance with the License. Please obtain a copy of the License at
 *  http://www.opensource.apple.com/apsl/ and read it before using this
 *  file.
 *  
 *  The Original Code and all software distributed under the License are
 *  distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
 *  EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
 *  INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
 *  Please see the License for the specific language governing rights and
 *  limitations under the License.
 *  
 *  @APPLE_LICENSE_HEADER_END@
 */


#include <sstream>

#include <security_cdsa_client/aclclient.h>
#include <Security/SecKey.h>
#include <Security/SecKeychainItem.h>

#include "CardOSRecord.h"

#include "Attribute.h"
#include "MetaAttribute.h"
#include "MetaRecord.h"

using namespace std;

//
// CardOSRecord
//

CardOSRecord::CardOSRecord(const Tokend::MetaRecord &metaRecord, CardOSAPI_X509Identity & ident)
  : identity(ident)
{
  desc = identity.getDescription();
}

CardOSRecord::~CardOSRecord()
{ 
}

CardOSAPI_X509Identity & CardOSRecord::getRecord()
{
  return identity;
}


//
// CardOSCertificateRecord
//

CardOSCertificateRecord::CardOSCertificateRecord(const Tokend::MetaRecord & metaRecord, CardOSAPI_X509Identity & ident)
  : CardOSRecord(metaRecord, ident)
{
  certData = identity.getCertificate();

  attributeAtIndex(metaRecord.metaAttribute(kSecLabelItemAttr).attributeIndex(),
                   new Tokend::Attribute(desc));
}


CardOSCertificateRecord::~CardOSCertificateRecord()
{ 
}


const char * CardOSCertificateRecord::description()
{
  return reinterpret_cast<const char *>(desc.c_str());
}


Tokend::Attribute * CardOSCertificateRecord::getDataAttribute(Tokend::TokenContext * tokenContext)
{    
  return new Tokend::Attribute(&certData[0], certData.size());
}


//
// CardOSKeyRecord
//

CardOSKeyRecord::CardOSKeyRecord(const Tokend::MetaRecord & metaRecord, CardOSAPI_X509Identity & ident)
  : CardOSRecord(metaRecord, ident)
{
  stringstream s;
  
  s << "PIN" << identity.getAuthId();
  pinIdStr = s.str();
  
  // Set key description attribute.
  attributeAtIndex(metaRecord.metaAttribute(kSecKeyPrintName).attributeIndex(), 
                   new Tokend::Attribute(desc));

  // Set key size attribute.
  attributeAtIndex(metaRecord.metaAttribute(kSecKeyKeySizeInBits).attributeIndex(),
                   new Tokend::Attribute((uint32)ident.getKeyLength()));
  attributeAtIndex(metaRecord.metaAttribute(kSecKeyEffectiveKeySize).attributeIndex(),
                   new Tokend::Attribute((uint32)ident.getKeyLength()));
  
  // Set key usage attribute.
  attributeAtIndex(metaRecord.metaAttribute(kSecKeyDecrypt).attributeIndex(),
                   new Tokend::Attribute(ident.canDecrypt()));
  attributeAtIndex(metaRecord.metaAttribute(kSecKeyUnwrap).attributeIndex(),
                   new Tokend::Attribute(ident.canUnwrap()));
  attributeAtIndex(metaRecord.metaAttribute(kSecKeySign).attributeIndex(),
                   new Tokend::Attribute(ident.canSign()));					 
}


CardOSKeyRecord::~CardOSKeyRecord()
{ 
}


size_t CardOSKeyRecord::getSizeInBits() const
{
  return identity.getKeyLength();
}


void CardOSKeyRecord::getAcl(const char *tag, uint32 &count, AclEntryInfo *&acls)
{
  if (!mAclEntries) {
    mAclEntries.allocator(Allocator::standard());
    
    // Anyone can read the DB record for this key (which is a reference CSSM_KEY)
    mAclEntries.add(CssmClient::AclFactory::AnySubject(mAclEntries.allocator()),
                    AclAuthorizationSet(CSSM_ACL_AUTHORIZATION_DB_READ, 0));
    
    // Setup the remainder of the acl based on the key type.
	mAclEntries.add(CssmClient::AclFactory::PinSubject(mAclEntries.allocator(), 1),
					AclAuthorizationSet(CSSM_ACL_AUTHORIZATION_SIGN,
										CSSM_ACL_AUTHORIZATION_DECRYPT, 0),
					pinIdStr.c_str());
#if 0	
	mAclEntries.addPin(CssmClient::AclFactory::PWSubject(Allocator::standard()), credential.getAuthId());
	mAclEntries.addPin(CssmClient::AclFactory::PromptPWSubject(Allocator::standard(), CssmData()), credential.getAuthId());
#endif
  }
  count = mAclEntries.size();
  acls = mAclEntries.entries();
}



