/*
 *  CardOSRecord.h
 *
 *  Copyright (c) 2011 Atos IT Solutions and Services GmbH
 *
 *  CardOS TokenD implementation based on XYZRecord.h provided 
 *  as part of the Apple TokenD developer information available from 
 *  dts@apple.com .
 *
 *  Use diff -c XYZRecord.h CardOSRecord.h to generate a patch.
 *
 *  See below for the copyright notice of the original file
 *  and the license applying to both the original and modified code.
 *
 *  Copyright (c) 2004 Apple Computer, Inc. All Rights Reserved.
 * 
 *  @APPLE_LICENSE_HEADER_START@
 *  
 *  This file contains Original Code and/or Modifications of Original Code
 *  as defined in and that are subject to the Apple Public Source License
 *  Version 2.0 (the 'License'). You may not use this file except in
 *  compliance with the License. Please obtain a copy of the License at
 *  http://www.opensource.apple.com/apsl/ and read it before using this
 *  file.
 *  
 *  The Original Code and all software distributed under the License are
 *  distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
 *  EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
 *  INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
 *  Please see the License for the specific language governing rights and
 *  limitations under the License.
 *  
 *  @APPLE_LICENSE_HEADER_END@
 */



#ifndef _CARDOSRECORD_H_
#define _CARDOSRECORD_H_

#include "Record.h"
#include "CardOSToken.h"


class CardOSToken;


class CardOSRecord : public Tokend::Record
{
  NOCOPY(CardOSRecord)
  
public:
  CardOSRecord(const Tokend::MetaRecord & metaRecord, CardOSAPI_X509Identity & ident);
  virtual ~CardOSRecord();
  virtual CardOSAPI_X509Identity & getRecord();
  
protected:
  CardOSAPI_X509Identity & identity;
  string desc;
};


class CardOSCertificateRecord : public CardOSRecord
{
  NOCOPY(CardOSCertificateRecord)
  
public:
  // Constructor
  CardOSCertificateRecord(const Tokend::MetaRecord & metaRecord, CardOSAPI_X509Identity & ident);
  
  // Destructor
  ~CardOSCertificateRecord();
  
  // Accessors
  virtual const char * description();
  virtual Tokend::Attribute * getDataAttribute(Tokend::TokenContext * tokenContext);
  
protected:
  BinaryObject certData;
};


class CardOSKeyRecord : public CardOSRecord
{
  NOCOPY(CardOSKeyRecord)
  
public:
  // Constructor
  CardOSKeyRecord(const Tokend::MetaRecord &metaRecord, CardOSAPI_X509Identity & ident);

  // Destructor
  ~CardOSKeyRecord();
  
  // Accessors
  virtual size_t getSizeInBits() const;
  virtual void getAcl(const char *tag, uint32 &count, AclEntryInfo *&acls);
  
private:
  string pinIdStr;
  AutoAclEntryInfoList mAclEntries;    
};

#endif /* _CARDOSRECORD_H_ */
