/*
 *  CardOSSchema.cpp
 *
 *  Copyright (c) 2011 Atos IT Solutions and Services GmbH
 *
 *  CardOS TokenD implementation based on XYZSchema.cpp provided 
 *  as part of the Apple TokenD developer information available from 
 *  dts@apple.com .
 *
 *  Use diff -c XYZSchema.cpp CardOSSchema.cpp to generate a patch.
 *
 *  See below for the copyright notice of the original file
 *  and the license applying to both the original and modified code.
 *
 *  Copyright (c) 2004 Apple Computer, Inc. All Rights Reserved.
 * 
 *  @APPLE_LICENSE_HEADER_START@
 *  
 *  This file contains Original Code and/or Modifications of Original Code
 *  as defined in and that are subject to the Apple Public Source License
 *  Version 2.0 (the 'License'). You may not use this file except in
 *  compliance with the License. Please obtain a copy of the License at
 *  http://www.opensource.apple.com/apsl/ and read it before using this
 *  file.
 *  
 *  The Original Code and all software distributed under the License are
 *  distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
 *  EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
 *  INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
 *  Please see the License for the specific language governing rights and
 *  limitations under the License.
 *  
 *  @APPLE_LICENSE_HEADER_END@
 */


#include "CardOSSchema.h"

#include "MetaAttribute.h"
#include "MetaRecord.h"

#include <Security/SecCertificate.h>
#include <Security/SecKeychainItem.h>
#include <Security/SecKey.h>

using namespace Tokend;



CardOSSchema::CardOSSchema() :
  mKeyAlgorithmCoder(uint32(CSSM_ALGID_RSA)),
  mKeySizeCoder(uint32(1024))
{
}


CardOSSchema::~CardOSSchema()
{
}


Tokend::Relation *CardOSSchema::createKeyRelation(CSSM_DB_RECORDTYPE keyType)
{
  Relation *rn = createStandardRelation(keyType);
  
  // Set up coders for key records.
  MetaRecord &mr = rn->metaRecord();
  mr.keyHandleFactory(&mCardOSKeyHandleFactory);
  
  // Print name of a key might as well be the key name.
  mr.attributeCoder(kSecKeyPrintName, &mDescriptionCoder);
  
  // Other key valuess
  mr.attributeCoder(kSecKeyKeyType, &mKeyAlgorithmCoder);
  mr.attributeCoder(kSecKeyKeySizeInBits, &mKeySizeCoder);
  mr.attributeCoder(kSecKeyEffectiveKeySize, &mKeySizeCoder);
  
  // Key attributes
  mr.attributeCoder(kSecKeyExtractable, &mFalseCoder);
  mr.attributeCoder(kSecKeySensitive, &mTrueCoder);
  mr.attributeCoder(kSecKeyModifiable, &mFalseCoder);
  mr.attributeCoder(kSecKeyPrivate, &mTrueCoder);
  mr.attributeCoder(kSecKeyNeverExtractable, &mTrueCoder);
  mr.attributeCoder(kSecKeyAlwaysSensitive, &mTrueCoder);
  
  // Key usage
  mr.attributeCoder(kSecKeyEncrypt, &mFalseCoder);
  mr.attributeCoder(kSecKeyWrap, &mFalseCoder);
  mr.attributeCoder(kSecKeyVerify, &mFalseCoder);
  mr.attributeCoder(kSecKeyDerive, &mFalseCoder);
  mr.attributeCoder(kSecKeySignRecover, &mFalseCoder);
  mr.attributeCoder(kSecKeyVerifyRecover, &mFalseCoder);
  
  return rn;
}

void CardOSSchema::create()
{
  Schema::create();
  
  createStandardRelation(CSSM_DL_DB_RECORD_X509_CERTIFICATE);
  createKeyRelation(CSSM_DL_DB_RECORD_PRIVATE_KEY);
  createKeyRelation(CSSM_DL_DB_RECORD_PUBLIC_KEY);
  createKeyRelation(CSSM_DL_DB_RECORD_SYMMETRIC_KEY);
  createStandardRelation(CSSM_DL_DB_RECORD_GENERIC);
}
